Class: M3u8::GapItem

Inherits:
Object
  • Object
show all
Includes:
Serializable
Defined in:
lib/m3u8/gap_item.rb

Overview

GapItem represents an EXT-X-GAP tag to indicate that the segment URI to which it applies does not contain media data and should not be loaded by clients.

Instance Method Summary collapse

Methods included from Serializable

#as_json, serialize, #to_h, #to_json

Instance Method Details

#to_sString

Render as an m3u8 EXT-X-GAP tag.

Returns:

  • (String)


12
13
14
# File 'lib/m3u8/gap_item.rb', line 12

def to_s
  '#EXT-X-GAP'
end